/*
  Maxwell's Lemur
  Copyright (c) 1997 Daniel Risacher <magnus@alum.mit.edu>
  
  Siag, Scheme In A Grid
  Copyright (C) 1996, 1997  Ulric Eriksson <ulric@edu.stockholm.se>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __CORE_H__
#define __CORE_H__

#include <glib.h>

#define __GUI__

#ifdef __GUI__
#include "../ngrid/ngrid.h"
#endif

#define VERSION "Maxwell's Lemur -|- pre-alpha"
#define FILE_VERSION "ascii1"

#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif

typedef enum 
{
  CELL_STATE_OLD,
  CELL_STATE_NEW,
  CELL_STATE_PENDING,
  CELL_STATE_CIRCULAR
} CellState;

typedef enum
{
  VALUE_TYPE_EMPTY,
  VALUE_TYPE_STRING,
  VALUE_TYPE_NUMBER,
  VALUE_TYPE_PIXMAP,
  VALUE_TYPE_WIDGET, 
  VALUE_TYPE_UNCALCULATED,
  VALUE_TYPE_ERROR
} ValueType;

typedef enum 
{
  CELL_TYPE_EMPTY,
  CELL_TYPE_EXPRESSION,
  CELL_TYPE_NUMBER,     /* not used yet */
  CELL_TYPE_STRING,     /* not used yet */
  CELL_TYPE_PIXMAP,     /* unimplemented */
  CELL_TYPE_WIDGET,     /* unimplemented */
  CELL_TYPE_BADREF      /* not used yet */
} CellType;

typedef enum
{
  JUST_TOP_LEFT,     JUST_TOP_CENTER,      JUST_TOP_RIGHT,
  JUST_CENTER_LEFT,  JUST_CENTER,          JUST_CENTER_RIGHT,
  JUST_BOTTOM_LEFT,  JUST_BOTTOM_CENTER,   JUST_BOTTOM_RIGHT,
  JUST_UNSPECIFIED
} CellJustification;

typedef struct {
  gint row, col;
} position;

typedef union {
  gchar *text;
  double number;
} cval;

typedef struct _CellFeatures {
  gulong                 text;       /* text property flags */
  CellJustification      just;
#ifdef __GUI__
  GtkNGridEntryFeatures *graphic;    /* graphical features */
#endif __GUI__
} CellFeatures;

typedef struct _cell {
  gchar *formula;              /* formula for cell */
  cval value;                  /* results of evaluating formula */
  guchar interpreter;          /* which interpreter to use */
  CellType ctype;               /* type of cell */
  ValueType vtype;               /* type of cell */
  CellState state;             /* update flag: see design.txt */
  CellFeatures *features;      /* cell-specific featurs or NULL */
} cell;

typedef struct _row {
  CellFeatures *default_features;  /* over-ridden by cell features */
  gint height;
  gint nent_cells;
  cell* cells;  
} row_desc;

typedef struct _col {
  CellFeatures *default_features;  /* over-ridden by cell and row features */
  gint width;
} col_desc;

typedef struct _buffer_page {
  gchar *name;      /* name of page */
  gint max_row;     /* index of highest row with data */
  gint max_col;     /* index of highest col with data */
  gint nent_rows;   /* number of entries in rows */
  row_desc* rows;   /* array nent_rows long; contains the data */
  gint nent_cols;   /* number of entries in cols */
  col_desc* cols;   /* array nent_cols long; contains format only */
  gint standard_width;
  gint standard_height;
  CellFeatures *default_features;  /* over-ridden by cell, row, & col */
  struct _buffer_page *next;
} buffer_page;

typedef struct _buffer {
  gchar *name;	/* identifies the buffer */
  gchar *path;	/* file name */
  gint dirty;   /* not used yet */
  buffer_page *pages;
  struct _buffer *next;	/* next buffer in buffer ring */
} buffer;


typedef struct _eval_struct {
  gint errorflag;
  gint siag_type;
  gint siag_row;
  gint siag_col;
  buffer *siag_buffer;
  struct _eval_struct *next;
} eval_struct;

#include "matrix.h"
#include "buffer.h"
#include "bpage.h"
#include "interpreters.h"
#include "fileio.h"
#include "guilei.h"


#endif /* __CORE_H__ */
